import playButton from '../assets/logo/play.png'
import pauseButton from '../assets/logo/pause.png'
import deleteButton from '../assets/logo/delete.png'
import editButton from '../assets/logo/edit.png'
import downloadButton from '../assets/logo/download.png'
import likeButton from '../assets/logo/like.png'
import likedButton from '../assets/logo/liked.png'
import backModal from '../assets/logo/backModal.png'
import { useNavigate } from 'react-router-dom'
import { useEffect, useState } from 'react'
import { useSongs } from '../contexts/SongContext'
import ReactModal from 'react-modal'
import { useCookies } from 'react-cookie'
import { IoAlertCircle, IoSend } from "react-icons/io5";
import { FileUploader } from 'react-drag-drop-files'
import { AiFillEdit,AiOutlineCheck,AiFillHeart,AiFillDownCircle,AiFillDelete } from "react-icons/ai";
import { BsMusicNoteBeamed } from "react-icons/bs";
import { BiSolidCommentDetail, BiLike } from "react-icons/bi";
import { useAlbums } from '../contexts/AlbumContext'
import TextareaAutosize from 'react-textarea-autosize';
import ClipLoader from "react-spinners/ClipLoader";
import axios from 'axios'
import useDebounce from '../customHooks/debounce'
export function Card(props){
    const [cookies,setCookies]=useCookies();
    const customStyles = {
        content: {
            backgroundColor:'#1C2540',
            whiteSpace: 'pre-line',
            borderRadius:'20px',
            display:'flex',
            flexDirection:'column',
            justifyContent:'space-between',
            alignItems:'center',
            textAlign:'center',
            color : '#FFFFFF',
            paddingBottom : '70px'
        },
      };

    const [liked,setLiked]=useState(false);
    const [isOpen,setIsOpen]=useState(false)
    const [isDelete,setIsDelete]=useState(false)
    const [isEdit,setIsEdit]=useState(false)
    const {isPlaying,setCurrSong,setIsPlaying,currentSong,deleteSong,updateSong}=useSongs();
    
    const [likeList,setLikeList] = useState();
    const [likeListLength,setLikeListLength] = useState();
    const [isLikeList,setIsLikeList] = useState(false);
    const [pageLike,setPageLike] = useState()
    async function fetchLikeList(){
        try {
            const res = await axios.post(import.meta.env.VITE_API_URL + "/like/getLike",
            {
                song_id:props.data.song_id
            },
            {
                headers:{
                    'Authorization': `Bearer ${cookies.user.token}`
                }
            }
            );
            setLikeList(res.data);
            setLikeListLength(res.data.length);
            const idx = res.data.findIndex( (loopVariable) => loopVariable.user.user_id === cookies.user.user_id);  
            if(idx !== -1){
                setLiked(true);
            }
        } catch (error) {
            console.log(error)
        }
    }
    useEffect(()=>{
        fetchLikeList();
    },[liked])
    const handleLike = async ()=>{
        if(liked){
            try {
                const res = await axios.post(import.meta.env.VITE_API_URL + "/like/unlikeSong",
                {
                    song_id:props.data.song_id
                },
                {
                    headers:{
                        'Authorization': `Bearer ${cookies.user.token}`
                    }
                }
                );
                setLikeListLength(likeListLength-1);
            } catch (error) {
                console.log(error)
            }
            setLiked(false)
        }else{
            try {
                const res = await axios.post(import.meta.env.VITE_API_URL + "/like/likeSong",
                {
                    song_id:props.data.song_id
                },
                {
                    headers:{
                        'Authorization': `Bearer ${cookies.user.token}`
                    }
                }
                );
                setLikeListLength(likeListLength+1);
            } catch (error) {
                console.log(error)
            }
            setLiked(true)
        }
    }
    const handlePlay = ()=>{
        if(currentSong){

            if(currentSong.song_id===props.data.song_id){
                if(isPlaying){
                    setIsPlaying(false);
                }
                else{
                    setCurrSong(props.data,props.mySong);
                    setIsPlaying(true);
                }
            }else{
                setCurrSong(props.data,props.mySong);
                if(currentSong){
                    props.audioRef.current.currentTime=0;
                }
                setIsPlaying(true);
            }
        }else{
            setCurrSong(props.data,props.mySong);
            if(currentSong){
                props.audioRef.current.currentTime=0;
            }
            setIsPlaying(true);
        }
    }

    const handleDelete = async ()=>{
        await deleteSong(props.data);
        setIsDelete(false);
    }

    const [isEditJudul,setIsEditJudul] = useState(null);

    const [album_id,setAlbum_id]=useState(props.data.album_id);
    const [judul,setJudul] = useState(props.data.judul);
    const [lyric,setLyric] = useState(props.data.lyric);

    const [image,setImage] = useState(props.data.image);
    const [audio,setAudio] = useState(props.data.image);

    const [file, setFile] = useState(null);
    const [fileAudio, setFileAudio] = useState(null);
    
    const [error,setError] = useState(null)
    
    const [filePreview,setFilePreview] = useState(null);
    const fileTypes = ["JPG", "PNG", "JPEG"];
    const fileTypesAudio = ["MP3"];
    const handleChange = (file) => {
        setFile(file);
        setFilePreview(URL.createObjectURL(file));
        
        
    };
    const handleChangeAudio = (file)=>{
        setFileAudio(file);
    }

    const navigate=useNavigate();
    const handleUpdate = async () => {
        try {
            const formData = new FormData();
            if(file){
                formData.append("file", file);
            }
            if(fileAudio){
                formData.append("fileAudio", fileAudio);
            }
            formData.append('song_id', props.data.song_id);
            formData.append('judul', judul);
            formData.append('album_id',album_id);
            formData.append('lyric', lyric);
            formData.append('oldImage', props.data.image);
            formData.append('oldAudio', props.data.audio);
            if(currentSong){
                if(currentSong.song_id === props.data.song_id){
                    props.audioRef.current.currentTime=0;
                }
            }
            await updateSong(formData,props.data,setImage,props.mySong);
            setIsEdit(false);

        } catch (error) {
            
        }
    }
    

    const {myAlbum,fetchMyAlbum} = useAlbums();

    useEffect(()=>{
        if(isEdit===true){
            fetchMyAlbum();
        }
    },[isEdit])

    const [isComment,setIsComment] = useState(false);
    const [isMax,setIsMax] = useState(false);
    const [isMaxMessage,setIsMaxMessage] = useState(false);
    const [commentList,setCommentList] = useState([]);
    const [comment,setComment] = useState(null);
    const [isLoading,setIsLoading] = useState(false);
    const [page,setPage] = useState(1);
    async function fetchCommentList(){
        if(!isMaxMessage && commentList.length >= (page-1) * 10){
            setPage(page+1);
            try {
                setIsLoading(true);
                const res = await axios.post(import.meta.env.VITE_API_URL + "/comment/getComment",
                {
                    song_id:props.data.song_id,
                    take:page*10
                },
                {
                    headers:{
                        'Authorization': `Bearer ${cookies.user.token}`
                    }
                }
                );
                if(res.data.length !== commentList){
                    setCommentList(res.data);
                    setIsLoading(false);
                }else{
                    setIsMax(true);
                    setIsLoading(false);
                    setIsMaxMessage('No more comment available');
                }
            } catch (error) {
                console.log(error)
            }
        }else{
            setIsMaxMessage('No more comment available');
        }
    }

    useEffect(()=>{
        if(isComment){
            fetchCommentList()
        }
    },[isComment])

    const fetchComment = async () => {
        if(comment){
            try {
                const res = await axios.post(import.meta.env.VITE_API_URL + "/comment/commentSong",
                {
                    song_id:props.data.song_id,
                    comment:comment
                },
                {
                    headers:{
                        'Authorization': `Bearer ${cookies.user.token}`
                    }
                }
                );
                setCommentList([...commentList,res.data]);
                setComment(null);
            } catch (error) {
                
            }
        }
    }

    const handleDeleteComment = async (comment_id)=>{
        try {
            const res = await axios.post(import.meta.env.VITE_API_URL + "/comment/deleteComment",
            {
                comment_id:comment_id
            },
            {
                headers:{
                    'Authorization': `Bearer ${cookies.user.token}`
                }
            }
            );
            setCommentList(commentList.filter((comment) => comment.comment_id !== res.data.comment_id));
        } catch (error) {
            console.log(error.message)
        }
    }

    const [searchComment,setSearchComment] = useState(null)
    const handleSearchComment = async ()=>{
        if(searchComment!=''){
            try {
                const res = await axios.post(import.meta.env.VITE_API_URL + "/comment/searchComments",
                {
                    search:searchComment,
                    song_id:props.data.song_id,
                },
                {
                    headers:{
                        'Authorization': `Bearer ${cookies.user.token}`
                    }
                }
                );
                setCommentList(res.data);
                setPage(1);
                setIsMax(true);
            } catch (error) {
                console.log(error.message);
            }
        }else{
            setIsMax(false);
            setPage(1);
            fetchCommentList();
        }
    }
    const debounceOnChangeComment = useDebounce(handleSearchComment);

    const [searchLike,setSearchLike] = useState()
    const handleSearchLike = async ()=>{
        if(searchLike!=''){

            try {
                const res = await axios.post(import.meta.env.VITE_API_URL + "/like/searchLikes",
                {
                    search:searchLike,
                    song_id:props.data.song_id
                },
                {
                    headers:{
                        'Authorization': `Bearer ${cookies.user.token}`
                    }
                }
                );
                setLikeList(res.data);
            } catch (error) {
                console.log(error.message);
            }
        }else{
            fetchLikeList()
        }
    }
    const debounceOnChangeLike = useDebounce(handleSearchLike);
    return(
        <div className="card" id="song">
            <img onClick={()=>{setIsOpen(true)}} loading="lazy" className={props.song?"cardIMG mx-auto cursor-pointer object-cover":"cardIMG mx-auto"} src={import.meta.env.VITE_API_URL+'/imgsong/'+ props.data.image}/>
            <div className='flex flex-row place-content-between items-center mt-2'>
                <h2 className="cardTitle font-bold text-2xl">{props.data.judul}</h2>
                <div className='flex flex-row items-center text-[17px] gap-1 text-[#1C2540 font-bold] cursor-pointer mr-[12px]' onClick={()=>{setIsLikeList(true)}}><AiFillHeart className='text-2xl text-red-500 hover:text-red-600'/>{likeListLength}</div>
            </div>
            <h4 className="cardAuthor ml-[2px]">{props.data.user.username} - {props.data.album.judul}</h4>
            <h5 className="cardAuthor ml-[2px]">{props.data.year}</h5>

            <div className="cardButtonCon">
                    <ReactModal
                        isOpen={isOpen}
                        contentLabel="Example Modal"
                        onRequestClose={() => setIsOpen(false)}
                        style={customStyles}
                        
                    >   
                        <img src={backModal} className='fixed top-10 left-10 ml-2 mt-2 cursor-pointer' onClick={()=>{setIsOpen(false)}}></img>
                        <p className='text-xl py-5 px-14 w-full'>
                            {props.data.lyric && props.data.lyric!=='null'? props.data.lyric : "No lyric available"}
                        </p>
                    </ReactModal>

                    <ReactModal
                        isOpen={isComment}
                        contentLabel="Example Modal"
                        onRequestClose={() => setIsComment(false)}
                        style={customStyles}
                        
                    >   
                
                        <img src={backModal} className='fixed top-10 left-10 ml-2 mt-2 cursor-pointer' onClick={()=>{setIsComment(false)}}></img>
                        <div className='w-full pt-6 pb-8 px-12 w-full overflow-auto max-w-screen'>
                            <h1 className='text-3xl font-bold'>Comments</h1>
                            <div className="search-container py-5 my-2">
                                <input id="searchBar" type="text" name="search" placeholder="Search..." className="search-input" onChange={(e)=>{
                                    debounceOnChangeComment();
                                    setSearchComment(e.target.value);
                                }}/>
                                <a className="search-btn">
                                        <i className="fas fa-search"></i>      
                                </a>
                            </div>
                            <div id='scrollableDiv' className='w-full overflow-y-scroll max-h-[90%]'> 
                                    {commentList && commentList.map((user, index) => (
                                        <div className='flex flex-row items-center my-3 place-content-between' key={index}>
                                            <div className='flex-row flex items-center gap-2'>
                                                <img loading="lazy" className='w-[50px] h-[50px] object-cover rounded-full' src={import.meta.env.VITE_API_URL+'/img/'+ user.user.profile_picture}/>
                                                <div className='text-left'>
                                                    <p className='font-bold'>{user.user.username}</p>
                                                    <p className='text-gray-400'>{user.comment}</p>
                                                </div>
                                            </div>
                                            { cookies.user.user_id === user.user_id ? <AiFillDelete onClick={()=>{handleDeleteComment(user.comment_id)}} className='hover:text-gray-300 mr-10'></AiFillDelete> : <></>}
                                        </div> 
                                    ))}
                                    <ClipLoader
                                        color={`#ffffff`}
                                        loading={isLoading}
                                        cssOverride={null}
                                        size={50}
                                        aria-label="Loading Spinner"
                                        data-testid="loader"
                                    />
                                    <div className='flex justify-center items-center'>
                                        {commentList && commentList.length !==0 && !isMaxMessage && !isMax && <AiFillDownCircle className='text-3xl my-2 hover:text-gray-400' onClick={()=>{
                                            fetchCommentList();
                                        }}></AiFillDownCircle> 
                                        }
                                        {isMaxMessage && <div className="outerCard text-white font-[Garamond] text-[40px] p-14 text-center">
                                            {isMaxMessage}
                                        </div>}
                                    </div>
                                    
                                    {commentList && commentList.length === 0 && <p className='text-2xl mt-10'>No comments found 😢</p>}
                                </div>
                        </div>
                            <div className='flex flex-row justify-center w-full gap-2 items-end mb-2'>
                                <TextareaAutosize className='w-full resize-none text-black rounded-[12px] px-2' placeholder='Comment' defaultValue={comment} onChange={(e)=>{
                                setComment(e.target.value)
                                }}></TextareaAutosize>
                                <IoSend className='mb-1 hover:text-gray-200' onClick={fetchComment}></IoSend>
                            </div>

                    </ReactModal>
                    <ReactModal
                        isOpen={isLikeList}
                        contentLabel="Example Modal"
                        onRequestClose={() => setIsLikeList(false)}
                        style={customStyles}
                        
                    >   
                        <img src={backModal} className='fixed top-10 left-10 ml-2 mt-2 cursor-pointer' onClick={()=>{setIsLikeList(false)}}></img>
                        <div className='py-8 px-14 w-full'> 
                        <h1 className='text-3xl font-bold'>Likes</h1>
                            <div className="search-container py-5 my-2">
                                <input id="searchBar" type="text" name="search" placeholder="Search..." className="search-input" onChange={(e)=>{
                                    debounceOnChangeLike();
                                    setSearchLike(e.target.value);
                                }}/>
                                <a className="search-btn">
                                        <i className="fas fa-search"></i>      
                                </a>
                            </div>
                            {likeList && likeList.map((user,index)=>(
                                <div key={index}>
                                    <div className='flex flex-row items-center w-full place-content-between gap-10 my-3'>
                                    <div className='flex flex-row items-center gap-10'>
                                        <img loading="lazy" className='w-[50px] h-[50px] min-w-[50px] min-h-[50px] object-cover rounded-full' src={import.meta.env.VITE_API_URL+'/img/'+ user.user.profile_picture}/>
                                        <div>
                                            <p className="font-bold text-2xl text-left">{user.user.username}</p>
                                            <p className="text-[12px] text-gray-400 text-left">Liked this.</p>

                                        </div>
                                    </div>
                                    </div>
                                </div>

                                ))}
                                {likeList && likeList.length === 0 && <p className='text-2xl my-10'>No user found 😢</p>}

                            </div>

                    </ReactModal>

                    <ReactModal
                        isOpen={isDelete}
                        onRequestClose={()=>{setIsDelete(false)}}
                        contentLabel="Example Modal"
                        className="w-full h-full blueColor bg-transparent flex justify-center items-center"   
                    >
                        <div className="blueColor w-[500px] h-[500px] rounded-[12px] flex justify-center items-center p-5">
                            <IoAlertCircle className="text-white text-9xl"></IoAlertCircle>
                            <label className="font-['Garamond'] text-white font-bold text-2xl">Hapus Lagu ?</label>

                            <div className="flex flex-row justify-center items-center my-5 gap-5">
                                <button className="font-['Verdana'] bg-white text-black hover:bg-gray-300 rounded-[20px] text-sm w-40 h-6" onClick={()=>{
                                    setIsDelete(false);
                                }}> <h3>Cancel</h3> </button>
                                <button className="font-['Verdana'] blueFont text-white hover:bg-blue-800 rounded-[20px] text-sm w-40 h-6" onClick={()=>{handleDelete()}}> <h3>Delete</h3> </button>
                            </div>
                        </div>
                    </ReactModal>
                    <ReactModal
                        isOpen={isEdit}
                        onRequestClose={()=>{setIsImage(false)}}
                        contentLabel="Example Modal"
                        className="w-full h-full blueColor bg-transparent flex justify-center items-center"   
                    >
                        <div className="blueColor w-[500px] h-[820px] rounded-[12px] flex justify-center items-center px-5">
                            <div className='w-full cursor-pointer'>
                                <img src={backModal} onClick={()=>{setIsEdit(false)}}></img>
                            </div>
                                <div className="w-full items-center justify-center flex">
                                <label className="font-['Garamond'] text-white font-bold text-3xl my-1">Image</label>
                                    <FileUploader className="w-1/2 h-1/2 border-white border-1 border rounded-[12px] flex p-1 blueFont flex" handleChange={handleChange} name="file" types={fileTypes}
                                        children={
                                            <div className="w-[250px] h-[250px] border-white border-1 border rounded-[12px] flex p-1 blueFont">
                                                {filePreview ? <img src={filePreview} className="rounded-[9px] w-full h-full object-cover"/> : <img src={import.meta.env.VITE_API_URL+'/imgsong/'+props.data.image} className="rounded-[9px] w-full h-full object-cover"/>}
                                            </div>
                                        }
                                    />
                                </div>
                                    <div className='w-full'>
                                        <label className="font-['Garamond'] text-white font-bold">Judul</label>
                                        <div className="flex flex-row items-center">
                                        {!isEditJudul?<input className="font-['Open_Sans'] w-full my-1 h-7 px-2 rounded-[12px]" defaultValue={props.data.judul} readOnly></input>:<input className="font-['Open_Sans'] w-full my-1 h-7 px-2 rounded-[12px]" defaultValue={props.data.judul} onChange={(e)=>{setJudul(e.target.value)}}></input>}
                                        {!isEditJudul?<AiFillEdit className="ml-2 text-white cursor-pointer" onClick={()=>{setIsEditJudul(true)}}/> : <AiOutlineCheck className="ml-2 text-white cursor-pointer" onClick={()=>{
                                            setIsEditJudul(false);
                                            }}/>}
                                        </div>
                                        <label className="font-['Garamond'] text-white font-bold">Lyric</label>
                                        <div className="flex flex-row place-content-between items-center gap-5">
                                            <textarea name="year" id='songYear' className="w-full h-40 mt-1 rounded-[12px] px-2 resize-none py-2" placeholder="Update lyric" onChange={(e)=>{setLyric(e.target.value)}}/>
                                        </div>
                                        
                                        <div className=" flex">
                                            <label className="font-['Garamond'] text-white font-bold">Album</label>
                                            <select name="album_id" id="option" className='text-gray-500 h-[40px] rounded-[12px] px-1' defaultValue={album_id}  onChange={(e)=>{
                                                setAlbum_id(e.target.childNodes[e.target.selectedIndex].id)}}>
                                                <option >Change Album</option>
                                                {
                                                    myAlbum.map((album,index)=>(<option key={index} id={album.album_id} defaultValue={album.album_id}>{album.judul}</option>))
                                                    
                                                }
                                            </select>
                                        </div>

                                        <label className="font-['Garamond'] text-white mt-1 font-bold">Audio</label>
                                        <div className="flex flex-row justify-center items-center my-1 w-full">
                                                <FileUploader handleChange={handleChangeAudio} name="file" types={fileTypesAudio}
                                                children={
                                                    <div className="w-[460px] h-[48px] border-[#ffffff] border-1 border border-dashed rounded-[6px] border-2 flex place-content-between items-center flex-row p-4 ">
                                                        <div className="flex flex-row h-full items-center">
                                                            <BsMusicNoteBeamed className="text-[#ffffff] mr-2"></BsMusicNoteBeamed>
                                                            {!fileAudio?<span className="text-xs text-white"><span className='underline-offset-2'>Upload</span> or drop a file right here</span>:<span className="text-xs text-white"><span>Uploaded Successfully!</span> Upload another?</span>}
                                                        </div>
                                                        <span className="text-xs text-white">{fileTypesAudio}</span>
                                                    </div>
                                                    
                                                }
                                                
                                            />
                                        </div> 

                                        <div className="flex flex-row justify-center items-center my-5 gap-5 w-full">
                                        <button className="font-['Verdana'] bg-white text-black hover:bg-gray-300 rounded-[20px] text-sm w-40 h-6" onClick={()=>{
                                            setFilePreview(null);
                                            setFile(null);
                                            setFileAudio(null);
                                            setIsEdit(false);
                                        }}> <h3>Cancel</h3> </button>
                                            <button className="font-['Verdana'] blueFont text-white hover:bg-blue-800 rounded-[20px] text-sm w-1/3 h-6" onClick={()=>{
                                                handleUpdate();

                                            }}> <h3>Save</h3> </button>
                                        </div> 
                                    </div>
                        </div>
                    </ReactModal>
                {(cookies.user.is_admin || (cookies.user.username === props.data.user.username)) && <div className="playButton" onClick={()=>{setIsDelete(true)}}><img src={deleteButton}></img></div>}
                {(cookies.user.username === props.data.user.username) && <div className="playButton" onClick={()=>{setIsEdit(true)}}><img src={editButton}></img></div>}
                    <a href={import.meta.env.VITE_API_URL + "/Audio/"+props.data.audio} download={props.data.judul} target='_blank'><div className="playButton"><img src={downloadButton}></img></div></a>
                {(cookies.user.username !== props.data.user.username) && <BiSolidCommentDetail className='text-[45px] cursor-pointer text-[#1C2540]' onClick={()=>{setIsComment(true)}}></BiSolidCommentDetail>}
                {(cookies.user.username !== props.data.user.username) && <div id="1" className="playButton" onClick={handleLike}><img src={liked?likedButton:likeButton}></img></div>}
                {currentSong?<div id="1" className="playButton" onClick={()=>{handlePlay()}}><img src={!(currentSong.song_id===props.data.song_id)?playButton:(!isPlaying?playButton:pauseButton)}></img></div>:<div id="1" className="playButton" onClick={()=>{handlePlay(props.index)}}><img src={playButton}></img></div>}
            </div>
        </div>
    )
}