import axios from "axios";
import {
    createContext,
    useEffect,
    useContext,
    useReducer,
    useCallback,
  } from "react";
import { useCookies } from "react-cookie";
import { useNavigate } from "react-router-dom";
  
  const SongContext = createContext();
  
  const initialState = {
    songs: [],
    isPlaying: false,
    currentSong: null,
    mySongs:[],
    isMax:false,
    isMySong:null,
    error: "",
  };

  function reducer(state, action) {
    switch (action.type) {
      case "loading":
        return { ...state};
  
      case "songs/loaded":
        return {
          ...state,
          isMax:false,
          songs: action.payload,
        };
      case "songs/loadedPage":
        return {
          ...state,
          isMax : true,
        };
  
      case "song/loaded":
        return { ...state, isPlaying: true, currentSong: action.payload, isMySong:action.isMySong};
      
      case "mySongs/loaded":
        return { ...state, mySongs: action.payload};
      case "mySongs/updated":
        return { ...state, mySongs: action.payload, songs:action.songs};
      
      case "song/isPlaying":
        return { ...state, isPlaying: action.payload };
      
      case "song/next":
        return { ...state, isPlaying: false, currentSong: action.payload, isMax:false};
      case "song/created":
        return {
          ...state,
          songs: [...state.songs, action.payload],
          mySongs: [...state.mySongs, action.payload],
        };
      case "song/deleted":
        return {
          ...state,
          songs: state.songs.filter((song) => song.song_id !== action.payload),
          mySongs: state.mySongs.filter((song) => song.song_id !== action.payload),
        };
  

      case "rejected":
        return {
          ...state,
          isPlaying: false,
          error: action.payload,
        };
  
      default:
        throw new Error("Unknown action type");
    }
  }
  
  function SongProvider({ children }) {
    const [{ songs, isPlaying, currentSong, isMySong, error, mySongs, isMax}, dispatch] = useReducer(
      reducer,
      initialState
    );

    const [cookies,setCookies,removeCookies] = useCookies();


      async function fetchSongsAll(page) {
        dispatch({ type: "loading" });
  
        try {
          const res = await axios.get(import.meta.env.VITE_API_URL + '/song/getSongs',
          {
            headers:{
              'Authorization': `Bearer ${cookies.user.token}`,
              'skip':(page)*10
            },
          }
          );

        dispatch({ type: "songs/loaded", payload: res.data });

        } catch {
          dispatch({
            type: "rejected",
            payload: "There was an error loading songs...",
          });
        }
      }
      async function fetchSongs(page) {
        dispatch({ type: "loading" });
  
        try {
          const res = await axios.get(import.meta.env.VITE_API_URL + '/song/getSongs',
          {
            headers:{
              'Authorization': `Bearer ${cookies.user.token}`,
              'skip':(page)*10
            },
          }
          );
          const newSongsUnfilter = songs.concat(res.data);
          const newSongs =  newSongsUnfilter.filter(
            (obj, index) =>
              newSongsUnfilter.findIndex((item) => item.song_id === obj.song_id) === index
          );
          if(newSongs.length !== songs.length){
            dispatch({ type: "songs/loaded", payload: newSongs });
          }else{
            dispatch({ type: "songs/loadedPage"});
          }
        } catch {
          dispatch({
            type: "rejected",
            payload: "There was an error loading songs...",
          });
        }
      }
    
    
    const setCurrSong = useCallback(
      async function setCurrSong(song,mySong) {
        try {
          dispatch({ type: "song/loaded", payload:song, isMySong:mySong});
        } catch {
          dispatch({
            type: "rejected",
            payload: "There was an error loading the song...",
          });
        }
      },
      [currentSong]
    );
    
    const setIsPlaying = useCallback(
      async function setIsPlaying(boolean) {
        try {
          dispatch({ type: "song/isPlaying", payload: boolean });
        } catch {
          console.log('error');
          dispatch({
            type: "rejected",
            payload: "There was an error loading the song...",
          });
        }
      },
      [isPlaying]
    );

    async function newSong(formData) { 
      try {
        dispatch({ type: "loading" });
        const res = await axios.post(import.meta.env.VITE_API_URL + '/song/addSong',
        formData,
        {
            headers:{
                'Content-Type': 'multipart/form-data',
                'Authorization': `Bearer ${cookies.user.token}`
            }
        }
        );

        dispatch({
          type: "song/created",
          payload: res.data,
        });
      } catch (error) {
        console.log(error)
        dispatch({
          type: "rejected",
          payload: error.message,
        });
      }
     }
    async function deleteSong(song) { 
      try {
        dispatch({ type: "loading" });

        const res = await axios.post(import.meta.env.VITE_API_URL + '/song/deleteSong',
        song,
        {
            headers:{
                'Authorization': `Bearer ${cookies.user.token}`
            }
        }
        );

        dispatch({
          type: "song/deleted",
          payload: song.song_id,
        });
      } catch (error) {
        dispatch({
          type: "rejected",
          payload: error.message,
        });
      }
     }
  
     async function fetchMySongs(){
      try {

        const res = await axios.get(import.meta.env.VITE_API_URL + '/song/userSong',
         {
           headers:{
             'Authorization': `Bearer ${cookies.user.token}`,
           }
         }
         );
         
         dispatch({
           type: "mySongs/loaded",
           payload: res.data,
         })
         
       } catch (error) {
        dispatch({
          type: "rejected",
          payload: error.message,
        });
     }
   }

   async function updateSong(formData,data,setImage,mySong){
    try {
      const res = await axios.post(import.meta.env.VITE_API_URL + '/song/updateSong',
      formData,
      {
          headers:{
            'Content-Type': 'multipart/form-data',
            'Authorization': `Bearer ${cookies.user.token}`
          }
      }
      );

      const songIdx = mySongs.findIndex( (loopVariable) => loopVariable.song_id === data.song_id);
      const updatedSong = res.data;
      const songIdx2 = songs.findIndex( (loopVariable) => loopVariable.song_id === data.song_id);      
      if(songIdx2!== -1){
        var newSongs=songs;
        newSongs[songIdx2]=updatedSong;
      }else{
        var newSongs=songs;
      }
      
      var mySongsNew=mySongs;
      mySongsNew[songIdx]=updatedSong;
      
      setImage(res.data.image);
      if(currentSong){
        if(currentSong.song_id===res.data.song_id){
            setCurrSong(res.data,mySong);
        }
    }
      dispatch({
        type: "mySongs/updated",
        payload: mySongsNew,
        songs:newSongs,
      })
    } catch (error) {
      
    }
   }

   async function searchSongs(params){
    try {
      
      const res = await axios.post(import.meta.env.VITE_API_URL + '/song/searchSongs',
      {
        search:params.search
      },
       {
         headers:{
           'Authorization': `Bearer ${cookies.user.token}`,
         }
       }
       );
       
       dispatch({
         type: "songs/loaded",
         payload: res.data,
       })
       
     } catch (error) {
      dispatch({
        type: "rejected",
        payload: error.message,
      });
   }
 }
   async function filterSongs(album){
    try {
      
      const res = await axios.post(import.meta.env.VITE_API_URL + '/song/filterSongs',
      {
        album_id:album
      },
       {
         headers:{
           'Authorization': `Bearer ${cookies.user.token}`,
         }
       }
       );
       
       dispatch({
         type: "songs/loaded",
         payload: res.data,
       })
       
     } catch (error) {
      dispatch({
        type: "rejected",
        payload: error.message,
      });
   }
 }
    return (
      <SongContext.Provider
        value={{
          songs,
          isPlaying,
          isMySong,
          currentSong,
          error,
          mySongs,
          isMax,
          setCurrSong,
          setIsPlaying,
          newSong,
          fetchSongs,
          deleteSong,
          fetchMySongs,
          updateSong,
          searchSongs,
          filterSongs,
          fetchSongsAll,
          // fetchSongsPage,
          // setNextSong
        }}
      >
        {children}
      </SongContext.Provider>
    );
  }
  
  function useSongs() {
    const context = useContext(SongContext);
    if (context === undefined)
      throw new Error("CitiesContext was used outside the CitiesProvider");
    return context;
  }
  
  export { SongProvider, useSongs };