import { useEffect, useState } from "react";
import { Navbar } from "../components/navbar";
import { useAlbums } from "../contexts/AlbumContext"
import { useSongs } from "../contexts/SongContext";
import { useNavigate } from "react-router-dom";
import { FileUploader } from "react-drag-drop-files";
import { BsMusicNoteBeamed } from "react-icons/bs";
import { useCookies } from "react-cookie";
import TextareaAutosize from 'react-textarea-autosize';
import { useAuth } from "../contexts/FakeAuthContext";


export function AddSong(){
    const {albums,myAlbum,fetchMyAlbum}=useAlbums();
    const {checkToken}=useAuth()
    const {newSong}=useSongs();
    const [cookies,setCookies]=useCookies();
    
    useEffect(()=>{
        async function checkTokenPage(){

            const status = await checkToken();
            if(status!==200){
                logout()
                navigate('/login')
            }
        }

        checkTokenPage();
    },[])
    
    useEffect(()=>{
        fetchMyAlbum();
    },[])
    
    const [judul,setJudul]=useState(null);
    const [file,setFile]=useState(null);
    const [fileAudio,setFileAudio]=useState(null);
    const [album_id,setAlbum_id]=useState(null);
    const [lyric,setLyric]=useState(null);
    const [error,setError]=useState(null);
    const navigate=useNavigate();
    
    const fileTypes = ["JPG", "PNG", "JPEG"];
    const fileTypesAudio = ["MP3"];
    const {songs} = useSongs();
    const [currentValues, setCurrentValues] = useState(false);

    const handleChange = (file)=>{
        setFile(file);
    }
    const handleChangeAudio = (file)=>{
        setFileAudio(file);
    }
    
    const handleUpload = async ()=>{
        const year = new Date().getFullYear();
    
        if(file !== null && judul && year && fileAudio !== null && album_id){
            setError(null);
            const formData = new FormData();
            try {
                formData.append('file', file);
                if(lyric !== null){
                    if(lyric.length < 1000){
                        throw new Error('Lyric minimal 2000 karakter');
                    }else{
                        
                        formData.append('lyric', lyric);
                    }
                }
                formData.append('judul', judul);
                formData.append('year', year);
                formData.append('album_id', album_id);
                formData.append('fileAudio', fileAudio);
                await newSong(formData);
                navigate('/');
            } catch (error) {
                setError(error.message);
            }
        }else{
            setError('Pastikan semua data yang wajib terisi')
        }
        
    }
    
    return(
        <>
        <Navbar></Navbar>
            <div className="flex justify-center items-center h-screen">
                <div className="boxauth2 mb-14 w-4/5 max-w-[500px]">
                    <header className="h-auth">Add Song</header>
                    <div className="w-full">
                        <label name="judul" className="label-auth">Title</label>
                        <input name="judul" id='songTitle' type="text" className="inputauth w-full" onChange={(e)=>{setJudul(e.target.value)}}/>
                    </div>
                    <div className="w-full">
                        <label className="label-auth">Author</label>
                        <input name="penyanyi" id='songAuthor' type="text" className="inputauth w-full" defaultValue={cookies.user.username} readOnly/>
                    </div>
                    <div className="w-full">
                        <label className="label-auth">Year</label>
                        <input name="year" id='songYear' type="text" className="inputauth w-full" defaultValue={new Date().getFullYear()} readOnly/>
                    </div>
                    <div className="w-full">
                        <label className="label-auth">Lyric</label>
                        <textarea name="year" id='songYear' className="input-auth medium resize-none py-2 w-full h-10" placeholder="Leave black if no lyric available" onChange={(e)=>{setLyric(e.target.value)}}/>
                    </div>
                    <div className="container-input-auth flex">
                        <label className="label-auth">Album</label>
                        <select name="album_id" id="option" defaultValue={album_id}  onChange={(e)=>{
                            setAlbum_id(e.target.childNodes[e.target.selectedIndex].id)}}>
                            <option defaultValue="undef"> -- select an option -- </option>
                            {
                                myAlbum ? myAlbum.map((album,index)=>(<option key={index} id={album.album_id} defaultValue={album.album_id}>{album.judul}</option>)) : null
                                
                            }
                        </select>
                    </div>

                    <div className="container-input-auth">
                            <label className="label-auth flex">Audio</label>
                            <FileUploader handleChange={handleChangeAudio} name="file" types={fileTypesAudio}
                            children={
                                <div className="w-full h-[48px] border-[#0658C2] border-1 border border-dashed rounded-[6px] border-2 flex place-content-between items-center flex-row p-4 ">
                                    <div className="flex flex-row h-full items-center">
                                        <BsMusicNoteBeamed className="text-[#0658C2] mr-2"></BsMusicNoteBeamed>
                                        {!fileAudio?<span className=" sc-fqkvVR kFhUBM text-xs"><span>Upload</span> or drop a file right here</span>:<span className=" sc-fqkvVR kFhUBM text-xs"><span>Uploaded Successfully!</span> Upload another?</span>}
                                    </div>
                                    <span className=" sc-fqkvVR kFhUBM">{fileTypesAudio}</span>
                                </div>
                            }
                        />
                    </div>
                    <div className="container-input-auth">
                            <label className="label-auth flex">Image</label>
                            <FileUploader handleChange={handleChange} name="file" types={fileTypes}/>
                    </div>
                    <div className="flex justofy-center items-center">
                        {error && <p className="text-red-500 text-md font-bold mt-0 text-mid mt-2">{error}</p>}
                        <button className="btn-auth my-5" id="submitForm" onClick={handleUpload}>Upload</button>
                    </div>
                </div>    
            </div>
        </>
    )
}