import { useEffect, useState } from "react";
import { CardList } from "../components/cardList";
import { Navbar } from "../components/navbar";
import { useAlbums } from "../contexts/AlbumContext";
import { useSongs } from "../contexts/SongContext";
import useDebounce from "../customHooks/debounce";
import { useAuth } from "../contexts/FakeAuthContext";
import addAlbum from '../assets/logo/addAlbum.png';
import { NavLink } from "react-router-dom";
import { useCookies } from "react-cookie";
export function AlbumPage(){
    const {albums,fetchAlbums,searchAlbums,myAlbum,fetchMyAlbum}=useAlbums();
    const [search,setSearch]=useState(null);
    const {checkToken} = useAuth();
    const [cookies,setCookies] = useCookies()
    useEffect(()=>{
        async function checkTokenAlbum(){

            const status = await checkToken();
            if(status!==200){
                logout()
                navigate('/login')
            }
        }

        checkTokenAlbum();
    },[])

    useEffect(()=>{
        fetchMyAlbum();
    },[])


    const handleSearch = async ()=>{
        const searchParams = {
            search : search
        }
        await searchAlbums(searchParams);
      }

      const debounceOnChange = useDebounce(handleSearch)
    return(
        <div className='mainPage overflow-y-scroll'>
        <Navbar></Navbar>

        <div className="mainCon">
            <div className="cardCon albumCon mt-[100px] pb-[200px]">
                <div className="search-container py-5">
                    <input id="searchBar" type="text" name="search" placeholder="Search..." className="search-input" onChange={(e)=>{
                        debounceOnChange();
                        setSearch(e.target.value);
                    }}/>
                    <a className="search-btn">
                            <i className="fas fa-search"></i>      
                    </a>
                </div>
            <h2 className="searchText font-bold text-3xl">Daftar Album</h2>
                <div id='reload'>
                    {albums && <CardList data={albums}></CardList>}
                </div>
                <div id='reload' className="py-10 pb-[200px]">
                    <div className="filterCon flex justify-center items-center mt-20">
                        <h2 className="searchText font-bold text-3xl">My Albums</h2>
                    </div>
                    {myAlbum? <CardList song={false} data={myAlbum}></CardList> : <div className="text-2xl w-1/2 h-1/2 rounded-[12px] blueFont text-white text-mid py-2 px-1">No User Album</div>}
                <div className="card addAlbum mx-auto mt-14">
                        <NavLink  to="/addAlbum">
                            <img src={addAlbum}/>
                        </NavLink>
                </div>
                </div>
            </div>
        </div>
        
    </div>
    )
}