import { Navbar } from "../components/navbar"
import { Player } from "../components/player"
import { CardList } from "../components/cardList";
import { useEffect, useRef, useState } from "react";
import { useCookies } from "react-cookie";
import { useSongs } from "../contexts/SongContext";
import axios from "axios";
import addAlbum from '../assets/logo/addAlbum.png'
import { useAlbums } from "../contexts/AlbumContext";
import useDebounce from "../customHooks/debounce";
import { NavLink, useNavigate } from "react-router-dom";
import ClipLoader from "react-spinners/ClipLoader";
import { useAuth } from "../contexts/FakeAuthContext";

export function Home(props){

    const {songs,currentSong,fetchSongs,fetchMySongs,mySongs,isPlaying,searchSongs,isMax,filterSongs,fetchSongsAll}=useSongs();
    const {checkToken} = useAuth()
    const {albums,fetchAlbums} = useAlbums();
    const [album_id,setAlbum_id] = useState(null);
    const {logout} = useAuth();
    const [search,setSearch] = useState();
    const [page,setPage] = useState(0);
    const [isLoading,setIsLoading] = useState(false);
    const [pagination,setPagination] = useState(true);
    const [cookies,setCookies,removeCookies] = useCookies();
    const navigate = useNavigate()
    useEffect(()=>{
        async function checkTokenHome(){
                
                if(cookies.user.token){
                    const status = await checkToken();
                    if(status!==200){
                        logout()
                        navigate('/login')
                    }
                }
        }
        checkTokenHome();

        if(songs.length === 0 ){
            fetchSongsAll(0);
        }
            
        fetchMySongs();
        fetchAlbums();
    },[])
    

    const loadSong = async ()=>{
        if(pagination){
            if(!isMax){
                const page_temp=page+1;
                setIsLoading(true);
                await fetchSongs(page_temp);

                setPage(page_temp);
                setIsLoading(false);
            }
        }
    }

      const handleSearch = async ()=>{
        const searchParams = {
            search : search
        }
        await searchSongs(searchParams);
        setPagination(false);
      }

      const handleFilter = async (data)=>{
        if(data !== ''){
            await filterSongs(data);
            setPagination(false);
        }else{
            fetchSongsAll(0);
            setPagination(true)
        }
      }

      const debounceOnChange = useDebounce(handleSearch);
    return (
        <div className='mainPage overflow-y-scroll'>
            <Navbar></Navbar>
            <div className="mainCon">
                <div className="cardCon pb-[200px] mt-[90px]">
                <div className="search-container py-5">
                    <input id="searchBar" type="text" name="search" placeholder="Search..." className="search-input" onChange={(e)=>{
                        debounceOnChange();
                        setSearch(e.target.value);
                    }}/>
                    <a className="search-btn">
                            <i className="fas fa-search"></i>      
                    </a>
                </div>

                    <div className="flex flex-row place-content-between mb-2">
                        <h2 className="searchText font-bold text-2xl">Now Playing : {currentSong? currentSong.judul:''}</h2>
                        <select className="shadow-inner blueFont text-white rounded-[20px] pr-10 pl-3 hover:text-gray-300 bg-[#243E73]" name="album_id" id="option" defaultValue={album_id}  onChange={(e)=>{
                            // setAlbum_id(e.target.childNodes[e.target.selectedIndex].id);
                            handleFilter(e.target.childNodes[e.target.selectedIndex].id);
                            }}>
                            <option defaultValue="undef">Albums</option>
                            {
                                albums ? albums.map((album,index)=>(<option key={index} id={album.album_id} defaultValue={album.album_id}>{album.judul}</option>)) : null    
                            }
                        </select>
                    </div>
                    <div id='reload'>
                        <CardList data={songs} song={true} mySong={false} audioRef={props.audioRef} setPage={setPage} page={page} loadSong={loadSong}></CardList>
                        <ClipLoader
                            color={`#ffffff`}
                            loading={isLoading}
                            cssOverride={null}
                            size={50}
                            aria-label="Loading Spinner"
                            data-testid="loader"
                        />
                    </div>

                    <div id='reload' className="py-10">
                        <div className="filterCon flex justify-center items-center mb-5">
                            <h2 className="searchText font-bold text-3xl">My Songs</h2>
                        </div>
                        <CardList className='mt-10' data={mySongs} mySong={true} song={true} audioRef={props.audioRef}></CardList>
                    </div>
                    <div className="outerCard">
                    <div className="card addAlbum mb-14">
                        <NavLink  to="/addSong">
                            <img src={addAlbum}/>
                        </NavLink>
                    </div>
                </div>
                </div>

            </div>
        </div>


    )
}